--[[
	Vampire game mode
	This game mode is pretty simple: The first player of each team becomes a VAMPIRE! All vampires
	get healed by 75% of ANY damage which is caused on ANY enemy in their turn. The damage gets added
	up during the turn and after the turn ends, they get 75% of this total damage.
	It does not matter if it is fall damage or an exploding mine which already was on the landscape
	or the weapon the vampire fires. Even an enemy sentry making damage on an enemy during a vampire's
	turn gives the vampire also health (lucky you!).
	The calculated number also will get rounded, of course.

	
	The percentage can be changed in vampires.settings.leechfactor. Just write any other value you wish there.
	(Divide your percentage by 100 first)
]]

--[[FIXME: This game mode may behave strange when you play multiple rounds, like a vampire gets health on round start. You can reload the script to fix this. (clicking the "weapons" button will do this, you don't need to restart server!)
]]

addgamemode("vampires","Vampires")

if vampires==nil then vampires={} end

vampires.vampires={}
vampires.settings={}
-- SETTINGS START HERE
vampires.settings.leechfactor=0.75		-- with this factor the caused damage will be multiplied to calculate vampires health gain 
-- SETTING END HERE

vampires.flare=loadgfx("flare0.bmp")
vampires.turndamage=0
vampires.player=nil
vampires.setup=false
setmidhandle(vampires.flare)

-- Default deathmatch mode does not need any Lua!

function vampires.is_vampire(playerid)
	for i=1,#vampires.vampires do
		v=vampires.vampires[i]
		if(v==playerid) then
			return true
		end
	end
	return false
end

function vampires_gm_draw()
	for v=1, #vampires.vampires do
		p=vampires.vampires[v]
		if(getplayerhealth(p)>0) then
			setblend(blend_light)
			setalpha(0.9)
			setscale(0.5,1.2)
			setcolor(255,0,0)
			setrotation(0)
			drawimage(vampires.flare,getplayerx(p),getplayery(p))
		end
	end
end

function vampires_gm_update()
	--[[ TODO: if DC finally implemented the function PREFIX_gm_start(), move this block (between then...end) 
		   to a function named "vampires_gm_start()" and wipe out the variable vampires.setup.]]
	if(getround()==1) then
		vampires.setup=false
	end
	if(vampires.setup~=true) then
		local players=playertable(0)
		local teams={}
		vampires.vampires={}
		for i=1,#players do
			local p=players[i]
			if(teams[getplayerteam(p)]==nil) then
				teams[getplayerteam(p)]={}
			end
			table.insert(teams[getplayerteam(p)],p)
		end	
		for t=0,7 do
			if(teams[t]~=nil) then
				table.insert(vampires.vampires,teams[t][1])
			end
		end
		vampires.setup=true
	end
	--[[ FIXME: if a vampire kills the last enemy and the game ends and then a new game is started
		    without re-loading this script (re-starting the server), in the new round this vampire
		    gains health for nothing!]]
	if(getframe()==0) then
		if(vampires.is_vampire(vampires.player)==true) then
			local healthsuck = vampires.round(vampires.turndamage*vampires.settings.leechfactor)
			if(healthsuck>0) then
				playerheal(vampires.player,healthsuck)
			end
		end
		vampires.turndamage=0
	else
		vampires.player=playercurrent()
	end
end

function vampires_gm_playerdamage(victim,damage)
	if(getplayeralliance(victim)~=getplayeralliance(playercurrent())) then
		vampires.turndamage = vampires.turndamage + damage
	end
	return nil
end

function vampires.round(num) 
	if num >= 0 then
		return math.floor(num+.5) 
	else
		return math.ceil(num-.5)
	end
end
